@@ -16,7 +16,7 @@ Methods
 --
 `static_page(file_path, path)` register a static page while the file is located under `file_path` and will serve `path`
 
-`register(func, path: str)` 
+`register(func, path: str)`
 
 `unregister(method: str, path: str)`unregister any page (static or dynamic). Only affect the `method`-path (GET / POST)
 
@@ -34,27 +34,30 @@ def hello_world(data):
 ```
 This will return "Hello World!" with a status code of 200, as plain text to the client
 """
-from serverly.utils import *
-from serverly.objects import Request, Response
-from serverly import default_sites
-from fileloghelper import Logger
-import serverly.stater
-from typing import Union
-from http.server import BaseHTTPRequestHandler, HTTPServer
-from functools import wraps
-import warnings
-import urllib.parse as parse
-import re
-import mimetypes
 import importlib
+import mimetypes
+import multiprocessing
+import re
+import urllib.parse as parse
+import warnings
+from functools import wraps
+from http.server import BaseHTTPRequestHandler, HTTPServer
+from typing import Union
+
+import serverly.stater
+from fileloghelper import Logger
+from serverly import default_sites
+from serverly.objects import Request, Response
+from serverly.utils import *
 
 description = "A really simple-to-use HTTP-server"
 address = ("localhost", 8080)
-name = "Serverly"
-version = "0.2.10"
+name = "serverly"
+version = "0.3.1"
 logger = Logger("serverly.log", "serverly", False, False)
 logger.header(True, True, description, fileloghelper_version=True,
               program_version="serverly v" + version)
+error_response_templates = {}
 
 
 class Handler(BaseHTTPRequestHandler):
@@ -71,7 +74,7 @@ class Handler(BaseHTTPRequestHandler):
             parsed_url = parse.urlparse(self.path)
             data_length = int(self.headers.get("Content-Length", 0))
             received_data = str(self.rfile.read(data_length), "utf-8")
-            request = Request("GET", parsed_url.path, dict(
+            request = Request("GET", parsed_url, dict(
                 self.headers), received_data, self.client_address)
             response = _sitemap.get_content(request)
             self.respond(response)
@@ -87,7 +90,7 @@ class Handler(BaseHTTPRequestHandler):
             parsed_url = parse.urlparse(self.path)
             data_length = int(self.headers.get("Content-Length", 0))
             received_data = str(self.rfile.read(data_length), "utf-8")
-            request = Request("POST", parsed_url.path, dict(
+            request = Request("POST", parsed_url, dict(
                 self.headers), received_data, self.client_address)
             response = _sitemap.get_content(request)
             self.respond(response)
@@ -96,14 +99,13 @@ class Handler(BaseHTTPRequestHandler):
         except Exception as e:
             serverly.stater.error(logger)
             logger.handle_exception(e)
-            raise e
 
     def do_PUT(self):
         try:
             parsed_url = parse.urlparse(self.path)
             data_length = int(self.headers.get("Content-Length", 0))
             received_data = str(self.rfile.read(data_length), "utf-8")
-            request = Request("PUT", parsed_url.path, dict(
+            request = Request("PUT", parsed_url, dict(
                 self.headers), received_data, self.client_address)
             response = _sitemap.get_content(request)
             self.respond(response)
@@ -119,7 +121,7 @@ class Handler(BaseHTTPRequestHandler):
             parsed_url = parse.urlparse(self.path)
             data_length = int(self.headers.get("Content-Length", 0))
             received_data = str(self.rfile.read(data_length), "utf-8")
-            request = Request("DELETE", parsed_url.path, dict(
+            request = Request("DELETE", parsed_url, dict(
                 self.headers), received_data, self.client_address)
             response = _sitemap.get_content(request)
             self.respond(response)
@@ -132,7 +134,7 @@ class Handler(BaseHTTPRequestHandler):
 
 
 class Server:
-    def __init__(self, server_address, webaddress="/", name="pyserver", description="A PyServer instance."):
+    def __init__(self, server_address, webaddress="/", name="serverly", description="A serverly instance."):
         """
         :param webaddress: the internet address this server is accessed by (optional). It will automatically be inserted where a URL is recognized to be one of this server.
         :type webaddress: str
@@ -229,6 +231,16 @@ class StaticSite:
         return content
 
 
+def _verify_user(req: Request):
+    try:
+        identifier = req.path.path.split("/")[-1]
+        import serverly.user.mail
+        serverly.user.mail.verify(identifier)
+        return Response(body="You're verified 🎉")
+    except KeyError:
+        return Response(404, body="Sorry, but the verification code seems to be invalid.")
+
+
 class Sitemap:
     def __init__(self, superpath: str = "/", error_page: dict = None):
         """
@@ -242,7 +254,8 @@ class Sitemap:
         check_relative_path(superpath)
         self.superpath = superpath
         self.methods = {
-            "get": {},
+            # for verifying users of serverly.user and serverly.user.mail
+            "get": {"^/verify/[\w0-9]+": _verify_user},
             "post": {},
             "put": {},
             "delete": {}
@@ -315,7 +328,7 @@ class Sitemap:
         elif callable(site):
             try:
                 content = site(request)
-            except TypeError:
+            except TypeError as e:  # makes debugging easier
                 try:
                     content = site()
                 except TypeError as e:
@@ -346,17 +359,13 @@ class Sitemap:
         return response
 
     def get_content(self, request: Request):
-        response_code = 500
-        text = ""
-        info = {}
         site = None
         response = None
         for pattern in self.methods[request.method].keys():
-            if re.match(pattern, request.path):
+            if re.match(pattern, request.path.path):
                 site = self.methods[request.method][pattern]
                 break
         if site == None:
-            response_code = 404
             site = self.error_page.get(404, self.error_page[0])
             response = self.get_func_or_site_response(
                 site, request)
@@ -376,20 +385,10 @@ _sitemap = Sitemap()
 
 
 def serves_get(path: str):
-    """When using this wrapper, please return a tuple with a dict containing 'response_code', and the content as a str.
+    """Decorator for registering a function for `path`, with method GET"""
+    logger.handle_exception(DeprecationWarning(
+        "serverly.serves_get() and serves_post() are deprecated since serverly v0.3.0. Please use serverly.serves('GET') instead."))
 
-    Example(s):
-
-    return ({'response_code': 200}, 'Hello World!')
-
-    You can also give more headers:
-
-    return ({'response_code': 200, 'Content-type': 'text/plain', 'Content-Length': 4}, '1234')
-
-    Or, if you'd like to type as little as you can:
-
-    return {'code':200},'Hello there!'
-    """
     def wrapper_function(func):
         _sitemap.register_site("GET", func, path)
         @wraps(func)
@@ -400,20 +399,10 @@ def serves_get(path: str):
 
 
 def serves_post(path: str):
-    """When using this wrapper, please return a tuple with a dict containing 'response_code', and the content as a str.
+    """Decorator for registering a function for `path`, with method POST"""
+    logger.handle_exception(DeprecationWarning(
+        "serverly.serves_get() and serves_post() are deprecated since serverly v0.3.0. Please use serverly.serves('POST') instead."))
 
-    Example(s):
-
-    return ({'response_code': 200}, 'Hello World!')
-
-    You can also give more headers:
-
-    return ({'response_code': 200, 'Content-type': 'text/plain', 'Content-Length': 4}, '1234')
-
-    Or, if you'd like to type as little as you can:
-
-    return {'code':200},'Hello there!'
-    """
     def wrapper_function(func):
         _sitemap.register_site("POST", func, path)
         @wraps(func)
@@ -424,20 +413,7 @@ def serves_post(path: str):
 
 
 def serves(method: str, path: str):
-    """When using this wrapper, please return a tuple with a dict containing 'response_code', and the content as a str.
-
-    Example(s):
-
-    return ({'response_code': 200}, 'Hello World!')
-
-    You can also give more headers:
-
-    return ({'response_code': 200, 'Content-type': 'text/plain', 'Content-Length': 4}, '1234')
-
-    Or, if you'd like to type as little as you can:
-
-    return {'code':200},'Hello there!'
-    """
+    """Decorator for registering a function for `path`, with `method`"""
     def wrapper_function(func):
         _sitemap.register_site(method, func, path)
         @wraps(func)
@@ -447,8 +423,8 @@ def serves(method: str, path: str):
     return wrapper_function
 
 
-def static_page(file_path, path):
-    """Register a static page while the file is located under `file_path` and will serve `path`"""
+def static_page(file_path: str, path: str):
+    """Register a static page where the file is located under `file_path` and will serve `path`"""
     check_relative_file_path(file_path)
     check_relative_path(path)
     site = StaticSite(path, file_path)
@@ -469,21 +445,53 @@ def unregister(method: str, path: str):
     _sitemap.unregister_site(method, path)
 
 
-def start(superpath: str = "/"):
-    """Start the server after applying all relevant attributes like address. `superpath` will replace every occurence of SUPERPATH/ or /SUPERPATH/ with `superpath`. Especially useful for servers orchestrating other servers."""
-    logger.autosave = True
+def _start_server(superpath: str):
     _sitemap.superpath = superpath
     _server = Server(address)
     _server.run()
 
 
+def start(superpath: str = '/', mail_active=False):
+    """Start the server after applying all relevant attributes like address. `superpath` will replace every occurence of SUPERPATH/ or /SUPERPATH/ with `superpath`. Especially useful for servers orchestrating other servers."""
+    try:
+        logger.autosave = True
+        args = tuple([superpath])
+        server = multiprocessing.Process(
+            target=_start_server, args=args)
+        if mail_active:
+            import serverly.user.mail
+            mail_manager = multiprocessing.Process(
+                target=serverly.user.mail.manager.start)
+            mail_manager.start()
+        server.start()
+    except KeyboardInterrupt:
+        try:
+            del _server
+            server.join()
+            mail_manager.join()
+        except Exception as e:
+            logger.handle_exception(e)
+
+
 def register_error_response(code: int, msg_base: str, mode="enumerate"):
-    """register an error response template for `code`including the message-stem `msg_base`and accepting *args as defined by `mode`
+    """register an error response template for `code` based off the message-stem `msg_base`and accepting *args as defined by `mode`
 
     Modes
     ---
     - enumerate: append every arg by comma and space to the base
     - base: only return the base message
+
+    Example
+    ---
+    ```
+    register_error_response(404, 'Page not found.', 'base'
+    ```
+    You can now get the 404-Response by calling `error_response(404)` -> Response(code=404, body='Page not found.')
+    Or in enumerate mode:
+    ```
+    register_error_response(999, 'I want to buy: ', 'enumerate')
+    ```
+    `error_response(999, 'apples', 'pineapples', 'bananas')` -> Response(code=9l9, body='I want to buy: apples, pineapples, bananas')
     """
     def enumer(msg_base, *args):
         result = msg_base + ', '.join(args)
@@ -514,6 +522,3 @@ def error_response(code: int, *args):
             f"No template found for code {str(code)}. Please make sure to register them by calling register_error_response.")
     except Exception as e:
         logger.handle_exception(e)
-
-
-error_response_templates = {}
