@@ -1,11 +1,34 @@
 import base64
 import collections.abc
+import datetime
 import json as jsonjson
+import urllib.parse
+import warnings
 from typing import Union
 
 from serverly.utils import get_http_method_type, guess_response_info
 
 
+class DBObject:
+    """Subclass this to implement a `to_dict` method which is required by serverly to pass an object as a response body"""
+
+    def to_dict(self):
+        d = {}
+        for i in dir(self):
+            if not i.startswith("_") and not i.endswith("_") and not callable(i) and i != "metadata" and i != "to_dict":
+                a = getattr(self, i)
+                # json-serializable
+                if type(a) == str or type(a) == int or type(a) == float or type(a) == dict or type(a) == list or type(a) == bool:
+                    d[i] = a
+                elif issubclass(type(a), DBObject):
+                    d[i] = a.to_dict()
+                elif isinstance(a, datetime.datetime):
+                    d[i] = a.isoformat()
+                else:
+                    d[i] = str(a)
+        return d
+
+
 class CommunicationObject:
     def __init__(self, headers: dict = {}, body: Union[str, dict, list] = ""):
 
@@ -32,7 +55,7 @@ class CommunicationObject:
         return self._body
 
     @body.setter
-    def body(self, body: Union[str, dict, list]):
+    def body(self, body: Union[str, dict, list, DBObject]):
         if type(body) == dict or type(body) == list:
             self._obj = body
             self._body = jsonjson.dumps(body)
@@ -42,12 +65,16 @@ class CommunicationObject:
                 self._obj = jsonjson.loads(body)
             except jsonjson.JSONDecodeError:
                 self._obj = None
+        elif issubclass(body.__class__, DBObject):
+            d = body.to_dict()
+            self._body = jsonjson.dumps(d)
+            self._obj = body
 
 
 class Request(CommunicationObject):
     """This is passed along to registered functions as a representation of what the client requested."""
 
-    def __init__(self, method: str, path: str, headers: dict, body: Union[str, dict], address: tuple):
+    def __init__(self, method: str, path: urllib.parse.ParseResult, headers: dict, body: Union[str, dict], address: tuple):
         super().__init__(headers, body)
 
         self.method = get_http_method_type(method)
@@ -58,10 +85,15 @@ class Request(CommunicationObject):
         for key, value in headers.items():
             if key.lower() == "authentication" or key.lower() == "authorization":
                 self.auth_type, user_cred = tuple(value.split(" "))
-                decoded = str(base64.b64decode(user_cred), "utf-8")
-                self.user_cred = tuple(decoded.split(":"))
-                self.user_name = self.user_cred[0]
-                self.user_password = self.user_cred[1]
+                if self.auth_type.lower() == "basic":
+                    decoded = str(base64.b64decode(user_cred), "utf-8")
+                    self.user_cred = tuple(decoded.split(":"))
+                elif self.auth_type.lower() == "bearer":
+                    self.user_cred = user_cred
+                else:
+                    self.user_cred = None
+                    warnings.warn(
+                        "Requested auth method not supported. Expected Basic or Bearer")
                 self.authenticated = True
         if not self.authenticated:
             self._set_auth_none()
@@ -70,8 +102,6 @@ class Request(CommunicationObject):
     def _set_auth_none(self):
         self.auth_type = None
         self.user_cred = None
-        self.user_name = None
-        self.user_password = None
 
     def __str__(self):
         s = f"{self.method.upper()}-Request from '{self.address[0]}:{str(self.address[1])}' for '{self.path}' with a body-length of {str(len(self.body))} and {str(len(self.headers))} headers."
